#pragma once

#include "filesys.h"
#include <ESP8266WebServer.h>
#include <stdarg.h>

#define LOG_STR "[HTTPServer]"


class HTTPServer : public ESP8266WebServer {
private:
    const char* rootDir = "/";
 
    bool addRootFileHandler();
    bool addRootFileHandler_ls();
    bool formatFS() {
        return format_fs();
    }
    void listRoot() {
        ls(rootDir);
    }
    void logf(const char *format, ...) {
        va_list args;
        va_start(args, format);
        Serial.print(LOG_STR);
        Serial.printf(format, args);
        va_end(args);
    }

public:
    HTTPServer(const int _port, const char* _rootDir) :
       ESP8266WebServer(_port), rootDir(_rootDir)
    { }
    ~HTTPServer() 
    {
        Serial.printf("[HTTPServer] shut down ...\n\r");
    }

    bool start() {
        if(!mount_fs())
            return false;
        Serial.printf("[HTTPServer] LittleFS mounted !\n\r");

        Serial.printf("[HTTPServer] root:\n\r");
        this->listRoot();
        Serial.printf("\n\r");

        // on("/compress.html", [this]() {
        //     File f = open("/compress.html.gz");
        //     if (!f) {
        //         Serial.println("Failed to open file for reading");
        //         send(404, "text/plain", "Failed to open file for reading");
        //     }
        //     else
        //     {
        //         Serial.println("Streaming /compress.html.gz to client");
        //         const String cont = "text/html";
        //         streamFile(f, cont);
        //     }
        //     f.close();
        // });
        // on("/comp.html", [this]() {
        //     File f = open("/comp.html.gz");
        //     if (!f) {
        //         Serial.println("Failed to open file for reading");
        //         send(404, "text/plain", "Failed to open file for reading");
        //     }
        //     else
        //     {
        //         Serial.println("Streaming /comp.html.gz to client");
        //         const String cont = "text/html";
        //         streamFile(f, cont);
        //     }
        //     f.close();
        // });       
        
        on("/terminal", [this]() {
            String log_msg = "terminal: millis: ";
            log_msg += millis();
            send(200, "text/plain", log_msg);
        });

        // default handler
        onNotFound([this]() {
            String message = "File Not Found\n\n";
            message += "URI: ";
            message += uri();
            message += "\nMethod: ";
            message += (method() == HTTP_GET) ? "GET" : "POST";
            message += "\nArguments: ";
            message += args();
            message += "\n";

            for (uint8_t i = 0; i < args(); i++) {
                message += " " + argName(i) + ": " + arg(i) + "\n";
            }
            send(404, "text/plain", message);
        });

        if( this->addRootFileHandler_ls() ){
            this->begin();
            Serial.printf("[HTTPServer] Server active on Port 80 !\n\r");
            return true;
        }
        Serial.printf("[HTTPServer] Not starting Server, something went wrong !\n\r");
        return false;
    }
};
